/* client.c */

/*
 
 *   Permission is granted to any individual to copy, use, and/or
 * distribute this software so long as the distribution includes
 * all the original unmodified files. No part of this software may
 * be used and/or sold for profit or used with any commercial product.

 * DISCLAIMER:
 *   This software comes with NO WARRANTIES of any kind. In no event
 * will the author be liable for any financial, physical, moral, and/or
 * mental damages incurred directly or indirectly by the use or intent
 * to use of this software.
 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "dfcss.h"
#include "sock.h"
#include "sockio.h"
#include "config.h"
#include "fileio.h"

#define CONFIGFILE			"client.cfg"
#define COMMANDFILE			"playtcp.bat"
#define MAXARGS             128

extern COMMAND[], EXTRA[];
extern char MODE[];
extern int PORT;
extern int SKILL;
extern int EPISODE;
extern int MISSION;
extern char MONSTERS[];


main(argc, argv)
int argc;
char *argv[];
{
#ifdef __TURBOC__
	tcp_Socket mysock;
	tcp_Socket *sd = &mysock;
#else
	int sd;
#endif
	char hostname[1024], localhost[1024];
	char serverID[IDLEN+1];
    char msg[MAXMSG+1], commandline[MAXMSG+1];
    int msglen;
    char *myargs[MAXARGS+1], *p;
	int port = PORT_NUMBER;
    int i, j;

	if (readconfig(CONFIGFILE) == -1)
		printf("Cannot open config file [%s]. Using defaults.\n", CONFIGFILE);

	if ((i = CheckParm("-port", argc, argv)) && (i < argc-1))
		port = atoi(argv[i+1]);

    if (gethostname(localhost,1024) == -1){
        perror("gethostname failed");
		exit(1);
	}

	if (argc < 2){
		printf("Server Name: [%s] ",localhost);
		gets(hostname);
    	if ( strlen(hostname) == 0 )
	    	strcpy(hostname, localhost);
	}
	else strcpy(hostname, argv[1]);

    printf("Connecting to [%s] in port[%d]...", hostname, port);
#ifdef __TURBOC__
    sock_init();
    if (SockOpen(sd, hostname, port) == -1) exit(2);
#else
    if ((sd = SockOpen(hostname, port)) == -1) exit(2);
#endif
	printf("connected.\n");
	
	if (SockRead(sd, serverID, IDLEN) < 0){
		printf("Cannot read serverID.\n");
		exit(2);
	}

    serverID[IDLEN] = '\0';
	printf("ServerID[%s]\n", serverID);
	if (strncmp(SERVERID, serverID, IDLEN)){
		printf("Incorrect Version.\n");
		exit(2);
	}

    if ((msglen = readfile(CONFIGFILE, msg, MAXMSG)) == -1)
		printf("Cannot open config file [%s]. Using defaults.\n", CONFIGFILE);

	msg[msglen] = '\0';
    if (ASockWrite(sd, msg) < 0) return(-1);

	do{
		fflush(stdout);
        if (ASockRead(sd, msg) < 0) return(-1);
		printf("%s", msg);
	} while (strcmp(msg, CONNECT));

	/* read the Server's parting message, usually for the log file */
	if (ASockRead(sd, msg) < 0) return(-1);
	printf("\n%s\n", msg);

	/* read the commandline parameters */
	if (ASockRead(sd, commandline) < 0) return(-1);
	close(sd);

    printf("\nRunning...\n");
    printf("[%s %s] %s\n", COMMAND, EXTRA, commandline);
    fflush(stdout);

    sprintf(msg, "%s %s %s", COMMAND, EXTRA, commandline);
	writefile(COMMANDFILE, msg, strlen(msg));

    sprintf(msg, "%s %s", EXTRA, commandline);
    for (i=0, j=0; msg[i] && j < MAXARGS; j++){
        /* look for a parameter */
        while (isspace(msg[i]) && msg[i]) i++;
        p = msg+i;

        /* skip white spaces */
        while (!isspace(msg[i]) && msg[i]) i++;
        msg[i] = '\0';

        if (*p){
             myargs[j] = (char *) malloc(sizeof(char)*(strlen(p)+1));
             strcpy(myargs[j], p);
             i++;
        }
	}
    myargs[j] = NULL;
    printf("(%d) params.\n", j);

    if (execvp(COMMAND, myargs) == -1){
		perror("Cannot execute command");
		exit(3);
	}
	exit(0);
}
