/* server.c */

/*
 
 *   Permission is granted to any individual to copy, use, and/or
 * distribute this software so long as the distribution includes
 * all the original unmodified files. No part of this software may
 * be used and/or sold for profit or used with any commercial product.

 * DISCLAIMER:
 *   This software comes with NO WARRANTIES of any kind. In no event
 * will the author be liable for any financial, physical, moral, and/or
 * mental damages incurred directly or indirectly by the use or intent
 * to use of this software.
 
 */


#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <sys/time.h>
#include "dfcss.h"
#include "sock.h"

#define TMPFILE		"cfg.tmp"

#define printIP(str, peer)	sprintf(str, "%d.%d.%d.%d",\
								(htonl(peer.sin_addr.s_addr) >> 24) & 0xFF,\
								(htonl(peer.sin_addr.s_addr) >> 16) & 0xFF,\
								(htonl(peer.sin_addr.s_addr) >>  8) & 0xFF,\
								(htonl(peer.sin_addr.s_addr)      ) & 0xFF)
     

#define CONFIGFILE				"server.cfg"
#define MAXCLIENTS				4

#define MESSAGE			"\n\
========================\n\
  THE DOOM FRAG SERVER  \n\
========================\n\
Server version[%s]\n\
You are node #%d of %d.\n"\
, SERVERID, nfd+1, numnodes


int sd, fd[10];
int nfd;

void ServerReset()
{
	int i;
	
	for (i=0; i<nfd; i++)
		close(fd[i]);
	nfd = 0;
	printf("\nConnection reset.\n");
	printf("Closing all connections.\n");
    signal(SIGPIPE, ServerReset);
}

void handler()
{
	close(sd);
	printf("\nServer_ShutDown.\n");
	exit(3);
}

main(argc, argv)
int argc;
char *argv[];
{
	int port = PORT_NUMBER, numnodes = 2;
	char msg[MAXMSG+1], commandline[MAXMSG+1];
	char IPaddress[MAXCLIENTS][4*4+10];
	struct sockaddr_in peer;
	int peersize = sizeof(peer);
	fd_set rfds, trfds;
	struct timeval tv;
	int maxfd;
	int i, j;

	if (readconfig(CONFIGFILE) == -1)
		printf("Cannot open config file [%s]. Using defaults.\n", CONFIGFILE);

	if ((i = CheckParm("-nodes", argc, argv)) && (i < argc-1))
		numnodes = MIN(MAXCLIENTS, atoi(argv[i+1]));

	if ((i = CheckParm("-port", argc, argv)) && (i < argc-1))
		port = atoi(argv[i+1]);

    printf("Listener_Start[%d] on port [0x%x]\n", numnodes, port);
    sd = SockOpen(NULL, port);
    if (sd == -1) exit(2);

	FD_SET(sd, &rfds);
	maxfd = sd +1;

    signal(SIGPIPE, ServerReset);
    signal(SIGINT, handler);
	while (1){

		nfd = 0;
		printf("\nListener_Listen[%d] port[%d]...", nfd, port);
		fflush(stdout);

		while (nfd<numnodes){

			tv.tv_sec = 1;

    		/* wait for and then accept a connect request */
    		if (listen(sd, 5) == -1){
         		perror("Listener: listen failed");
				break;
			}

			trfds = rfds;
			if (select(maxfd, &trfds, NULL, NULL, &tv) < 0){
				perror("Listener: select");
				break;
			}

			if (FD_ISSET(sd, &trfds)){
    			if ((fd[nfd] = accept(sd, NULL, NULL)) == -1){
         			perror("Listener: accept failed");
					continue;
				}
			}
			else{
				for (i=0; i<nfd; i++)
                    if (ASockWrite(fd[i], ".") < 0) ServerReset();
				continue;
			}

			printf("connect.\n");
			fflush(stdout);

			printf("Listener_WriteServerID[%d]...", nfd);
			if (SockWrite(fd[nfd], SERVERID, IDLEN) < 0)
				continue;
			printf("done.\n");

			printf("Listener_ReadOptions...");
			/* read options from client */
            if (ASockRead(fd[nfd], msg) < 0){
				ServerReset();
				continue;
			}

			/* set variables from read msg */
			if (nfd == 0){
				writefile(TMPFILE, msg, strlen(msg));
				readconfig(TMPFILE);
			}
			printf("done.\n");
			/* printf("%s\n", msg); fflush(stdout); */

			getpeername(fd[nfd], (struct sockaddr *)&peer, &peersize);
			printIP(IPaddress[nfd], peer);
			if (!strcmp(IPaddress[nfd], "127.0.0.1")){
				getsockname(fd[nfd], (struct sockaddr *)&peer, &peersize);
				printIP(IPaddress[nfd], peer);
			}

			sprintf(msg, MESSAGE);
            if (ASockWrite(fd[nfd], msg) < 0){
				ServerReset();
				continue;
			}

            if (ASockWrite(fd[nfd], "Waiting for node(s)...") < 0){
				ServerReset();
				continue;
			}
			
			nfd++;
			printf("Listener_Listen[%d] port[%d]...", nfd, port);
			fflush(stdout);
		}

		sprintf(commandline, "%s -port %d -skill %d -warp %d %d %s ",
			MODE, PORT, SKILL, EPISODE, MISSION, MONSTERS);

		if (TIMER){
			sprintf(msg, "-timer %d", TIMER);
			strcat(commandline, msg);
		}

		for (i=0; i<numnodes; i++){
            if (ASockWrite(fd[i], CONNECT) < 0){
				ServerReset();
				continue;
			}
			if (ASockWrite(fd[i], "Server_Goodbye.") < 0){
				ServerReset();
				continue;
			}
			sprintf(msg, "%s -net %d ", commandline, i+1);
			for (j=0; j<numnodes; j++){
				if (i-j){
					strcat(msg, IPaddress[j]);
					strcat(msg, " ");
				}
			}
            if (ASockWrite(fd[i], msg) < 0){
				ServerReset();
				continue;
			}
		}

		for (i=0; i<numnodes; i++)
			close(fd[i]);

		printf("%d nodes Connected.\n", numnodes);
	}
}

